/*
 psik - program to compute path sequences of graphs
  version 3 (09.11.2015)

  authors:  J. Zygadlo (Jakub.Zygadlo@ii.uj.edu.pl)
            S. Bakalarski (Slawomir.Bakalarski@uj.edu.pl)

            Institute of Computer Science and Computational Mathematics,
            Faculty of Mathematics and Computer Science,
            Jagiellonian University

 Feel free to use and modify this program, but if you obtain
 interesting results with it - please let us know!



usage: psik input_file output_file1 output_file2

input_file format:
 line 1: number_of_vertices (common to all graphs in the file)
 line 2: number_of_graphs_in_the_file (nGraphs)
 line 3: adjacency_matrix_1
 ...
 line nGraphs+2: adjacency_matrix_nGraphs

matrices are symmetric and in Mathematica format, i.e. for a complete graph on 3 vertices: {{0,1,1},{1,0,1},{1,1,0}}

output_file1 contains adjacency matrix and corresponding path sequence for all graphs from input file

output_file2 contains only path sequences for all graphs from input file (one sequence per line)
*/


#include <iostream>
#include <fstream>
#include <string>
#include <ctime>
#include "graph.h"
#include "komb.h"
using namespace std;

int main(int argc, char **argv)
{
  int nGraphs;                   // total number of graphs in a file
  int nVertices;                 // number of vertices of a single graph
  ifstream fileIn;               // input file
  ofstream fileOut,fileOut2;     // output files
  Graph *g;                      // currently processed graph
  string s;                      // contains current line of input file
  
  if (argc!=4)                   // 3 arguments needed: input_file out_file seq_file
  {
    cerr<<"Usage: psik file.in file.out file.seq"<<endl;
    return 1;
  }
  
  fileIn.open(argv[1]);          // open input file
  if(fileIn.fail())
  {
    cerr<<"Failed to open file "<<argv[1]<<endl;
    return 2;
  }
  fileOut.open(argv[2]);         //open (create) first output file
  if(fileOut.fail())
  {
    cerr<<"Failed to create file "<<argv[2]<<endl;
    return 3;
  }
  fileOut2.open(argv[3]);        //open (create) second output file
  if(fileOut2.fail())
  {
    cerr<<"Failed to create file "<<argv[3]<<endl;
    return 3;
  }
  
  fileIn>>nVertices;             // read the number of vertices of a graph (common to all graphs in the file)
  if(nVertices>MAX_SIZE)
  {
    cerr<<"Graph size too large - only graphs up to "<<MAX_SIZE<<" vertices are allowed."<<endl;
    return 4;
  }
  fileIn>>nGraphs;               //  and the number of graphs
  if(!fileIn.good())
  {
    cerr<<"Wrong file "<<argv[1]<<endl;
    return 5;
  }
  
  cout<<"Found "<<nGraphs<<" graphs on "<<nVertices<<" vertices."<<endl;
  getline(fileIn,s);

  clock_t start = clock();

/*** VERSION 2 - uncomment next lines: from here ***/
/*
  cout<<"Generating sequences...";
  int*** setArray = new int**[nVertices+1];   //possible subset sizes: 0..nVertices
  for(int k=1; k<=nVertices; k++){
      setArray[k]=new int*[binom(nVertices, k)];  //array of k-element subsets of 1..nVertices
      for(int i=0; i<binom(nVertices, k); i++)
          setArray[k][i] = new int[k];         //every subset has k elements
      genComb(nVertices, k, setArray[k]);      //generate subsets (combinations) and save them to setArray[k]
  }
  int*** permArray = new int**[nVertices+1];  //possible sequence lengths: 0..nVertices
  for(int k=1; k<=nVertices; k++){
      permArray[k]=new int*[fact(k)*binom(nVertices, k)];  //array of k-element sequences of 1..nVertices
      for(int i=0; i<fact(k)*binom(nVertices, k); i++)
          permArray[k][i] = new int[k];        //every sequence has k elements
      for(int j=0; j<binom(nVertices, k); j++)                 //for every k-element subset setArray[k][j]
          permute(setArray[k][j], k, permArray[k]+j*fact(k));  // generate its permutations and save them to a range of fact(k) indices of permArray[k]
  }
  cout<<"done"<<endl;
*/
/*** to here - VERSION 2 ***/

  for (int i=1;i<=nGraphs;i++)  //for every graph in the file
  {
    if(100*i/nGraphs>100*(i-1)/nGraphs)	                    //if current graph increases the percentage of processed graphs
      cout<<"Processing graph "<<i<<" of "<<nGraphs<<"..."; // write the message
   
    getline(fileIn,s);          //read line from input file
    if(fileIn.fail())
    {
      cerr<<"Error reading file "<<argv[1]<<endl;
      return 5;
    }
    g=new Graph(s,nVertices);   //create graph from s
    
//calculate the path sequence
/*** VERSION 1: uncomment line below ***/
//    g->calculatePathSequence();
/*** VERSION 2: uncomment line below AND lines marked VERSION 2 above! ***/
//    g->calculatePathSequence2(setArray, permArray);
/*** VERSION 3: best suited for a single graph, admits larger number of vertices ***/
    g->calculatePathSequence3();
    
    fileOut<<s<<"\n";   //copy line from input to output file (adjacency matrix)
    for (int j=1;j<=nVertices-1;j++)
    {
      fileOut<<g->PathSequence[j]<<" ";         // save path numbers to both output files
      fileOut2<<g->PathSequence[j]<<" ";        //  one number at a time
    }
    fileOut<<g->PathSequence[nVertices]<<endl;  // save the last number (psi_n)
    fileOut2<<g->PathSequence[nVertices]<<endl;
    fileOut.flush();
    fileOut2.flush();
    
    if(100*i/nGraphs>100*(i-1)/nGraphs)    //if current graph increases the percentage of processed graphs
      cout<<(100*i/nGraphs)<<"% complete, time elapsed: "<<(clock()-start)/(float)CLOCKS_PER_SEC<<" seconds "<<endl;  //write percentage and time elapsed information
      
    delete g;                           // delete the graph
  }
  cout<<"Calculation complete.\n";      // goodbye message 
  
  // close all files
  fileIn.close();
  fileOut.close();
  fileOut2.close();

  return 0;
}

